import os
import shutil
import zipfile
from cStringIO import StringIO
from urlparse import urlparse
import cPickle

try:
    import elementtree.ElementTree as ET
except ImportError:
    try:
        import xml.etree.ElementTree as ET
    except ImportError:
        ET = None


from quixote import get_publisher

from qommon.storage import StorableObject
from qommon.misc import simplify, get_provider_key

from coltypes import ColType

class Inst(StorableObject):
    _names = 'insts'

    name = None
    url = None
    type_id = None

    def store(self):
        self.build()
        StorableObject.store(self)

    def build(self):
        coltype = ColType.get(self.type_id)

        url_template = get_publisher().URL_TEMPLATE
        self.url = url_template % simplify(self.name)

        if self.url.endswith('/'):
            url_stripped = self.url[:-1]
        else:
            url_stripped = self.url

        host, path = urlparse(self.url)[1:3]
        if path.endswith('/'):
            path = path[:-1]

        coldir = host
        if path:
            coldir += path.replace('/', '+')

        try:
            import wcs.wcs_cfg
            wcs_app_dir = wcs.wcs_cfg.APP_DIR
        except (ImportError, AttributeError):
            import wcs.Defaults
            wcs_app_dir = wcs.Defaults.APP_DIR

        col_inst_dir = os.path.join(wcs_app_dir, coldir)

        if os.path.exists(col_inst_dir):
            shutil.rmtree(col_inst_dir) # XXX: or abort?

        os.mkdir(col_inst_dir)

        fd = StringIO(coltype.settings)
        z = zipfile.ZipFile(fd, 'r')

        for f in z.namelist():
            path = os.path.join(col_inst_dir, f)
            data = z.read(f)
            if not os.path.exists(os.path.dirname(path)):
                os.mkdir(os.path.dirname(path))
            open(path, 'w').write(data)
        z.close()

        idp_metadata = ET.parse(file(os.path.join(get_publisher().data_dir, 'idp-metadata.xml')))
        entity_id = idp_metadata.getroot().attrib['entityID']
        idp_key = get_provider_key(entity_id)

        wcs_cfg = cPickle.load(file(os.path.join(col_inst_dir, 'config.pck')))
        wcs_cfg['identification'] = {'methods': ['idp']}
        wcs_cfg['idp'] = {
            idp_key: {
                'metadata': 'provider-%s-metadata.xml' % idp_key,
                'metadata_url': entity_id,
                'publickey_url': None,
                'role': 2}}
        wcs_cfg['sp'] = {
            'base_url': '%s/liberty' % url_stripped,
            'common_domain': None,
            'common_domain_getter_url': None,
            'metadata': 'metadata.xml',
            'organization_name': '%s au Quotidien' % self.name,
            'privatekey': 'private-key.pem',
            'providerid': '%s/liberty/metadata' % url_stripped,
            'publickey': 'public-key.pem',
            'saml2_base_url': '%s/saml' % url_stripped,
            'saml2_metadata': 'saml2-metadata.xml',
            'saml2_providerid': '%s/saml/metadata' % url_stripped}
        cPickle.dump(wcs_cfg, file(os.path.join(col_inst_dir, 'config.pck'), 'w'))

        idff_metadata_template = file(
                os.path.join(get_publisher().data_dir, 'idff-metadata-template')).read()
        file(os.path.join(col_inst_dir, 'metadata.xml'), 'w').write(
                idff_metadata_template.replace('[URL]', url_stripped))

        saml2_metadata_template = file(
                os.path.join(get_publisher().data_dir, 'saml2-metadata-template')).read()
        file(os.path.join(col_inst_dir, 'saml2-metadata.xml'), 'w').write(
                saml2_metadata_template.replace('[URL]', url_stripped))

        file(os.path.join(col_inst_dir, 
                    'provider-%s-metadata.xml' % idp_key),
                'w').write(
                    file(os.path.join(get_publisher().data_dir, 'idp-metadata.xml')).read())
        file(os.path.join(col_inst_dir, 'public-key.pem'), 'w').write(
                file(os.path.join(get_publisher().data_dir, 'public-key.pem')).read())
        file(os.path.join(col_inst_dir, 'private-key.pem'), 'w').write(
                file(os.path.join(get_publisher().data_dir, 'private-key.pem')).read())

        print >> file(get_publisher().APACHE_FILE_NAME, 'a'), \
                get_publisher().APACHE_LINE_TEMPLATE % simplify(self.name)

        fd = None
        try:
            fd = os.popen('sudo -S /etc/init.d/apache2 reload', 'w')
            print >> fd, 'plop' # this is not the password
        except Exception, e:
            pass
        if fd:
            try:
                fd.close()
            except IOError:
                pass

