import os

from quixote import get_publisher

from qommon import get_cfg
from qommon.form import *
from qommon.storage import StorableObject

class User(StorableObject):
    _names = 'users'

    name = None
    email = None
    is_admin = False

    name_identifiers = None
    lasso_dump = None

    def __init__(self, name = None):
        StorableObject.__init__(self)
        self.name = name
        self.name_identifiers = []

    def get_display_name(self):
        if self.name:
            return self.name
        if self.email:
            return self.email
        return _('Unknown User')
    display_name = property(get_display_name)

    def set_attributes_from_formdata(self, data):
        identities_cfg = get_cfg('identities', {})
        self.name = data.get('name')
        self.email = data.get('email')

    def get_formdef(cls):
        class WcsInstUserField:
            def __init__(self, id, label):
                self.id = id
                self.label = label

            def get_view_value(self, v):
                return v

        class WcsInstUserFields:
            fields = [WcsInstUserField('name', _('Name'))]
            def add_fields_to_form(self, form):
                for f in self.fields:
                    form.add(StringWidget, f.id, title = f.label, required = True)

            def get_data(self, form):
                d = {}
                for attr in ('name',):
                    d[attr] = form.get_widget(attr).parse()
                return d
        return WcsInstUserFields()
    get_formdef = classmethod(get_formdef)

    def get_accounts(self):
        import qommon.ident

        ident_methods = get_cfg('identification', {}).get('methods', [])

        d = {}
        for klass in [x for x in qommon.ident.get_method_classes() if x.key in ident_methods]:
            if klass.method_admin_widget:
                value = klass().get_value(self)
                d[klass.key] = value
        return d
